with ada.strings.unbounded;
use ada.strings.unbounded;
with Grammatiken; use Grammatiken;

package Baum is

   -- Zur Modellierung von Syntaxbumen:
   -- ----------------------------------  
   --
   -- Ein Syntaxbaum B bezieht sich auf eine gegebene Grammatik G;
   -- Der Baum ist ein Geflecht von Knoten, das von einem Knoten
   -- wurzel ausgeht. 
   
   type Knoten_Rec;
   
   type Knoten is access Knoten_Rec;
   
   type Syntaxbaum_Rec;
   
   type Syntaxbaum is access Syntaxbaum_Rec;
   
   type Syntaxbaum_Rec is record 
      G       : Grammatik;
      wurzel  : Knoten; 
   end record;
   
   -- Ein Knoten K modelliert entweder ein terminales oder ein
   -- nichtterminales Symbol der Grammatik.
   
   type Art is (T, NT);
   
   -- Jeder Knoten verweist auf den gesamten Syntaxbaum; dadurch
   -- ist Zugang zur Grammatik G und damit u.a. zu den Namen der
   -- Grammatiksymbole gegeben (ergibt sich als G.text(code)).
   -- Weitere Grammatikinformationen werden fr das Drucken, das
   -- Speichern und Lesen von Bumen nicht bentigt. 
   --
   -- Jeder Knoten hat direkte Verweise auf den Vaternoten sowie
   -- die linken und rechten Geschwister; bei einem NT-Knoten
   -- kommt der Verweis auf das erste Kind hinzu sowie die Anzahl
   -- der Kinder; in einem T-Knoten ist Platz fr die komplette
   -- Token-Information aus der lexikalischen Analyse (d.h. der
   -- Tokentext und die Position in der Eingabedatei).
   --
   -- Die in NT-Knoten gespeicherte Alternativen-Nummer sowie die
   -- eindeutige Prordnungsnummerierung smtlicher Knoten wird
   -- ggfs die Attributierung erleichtern.
   
   type Knoten_Rec (form : Art := NT) is record 
      baum   : Syntaxbaum;
      code   : Positive := 1;
      vater  : Knoten := null; 
      vor    : Knoten := null;
      nach   : Knoten := null;
      prae   : Positive := 1;
      case Form is
         when NT => 
            linkesKind  : Knoten := null;
            anzKinder   : Natural := 0;
            alternative : Natural := 0;
         when T => 
            scanText   : Unbounded_String;
            scanZeile  : Natural := 0;
            scanSpalte : Natural := 0;
      end case;
   end record;
   
   ----------------------------------------------------------------

--   -- Syntaxbaum als Textgraphik ausgeben
--   procedure druckeBaum (baum : Syntaxbaum);
--   
--   -- Syntaxbaum als Text speichern (je Knoten prae-Nummer, code, ...)
--   procedure speichereBaum (baum : Syntaxbaum; datei : out Textdatei);
--   
--   -- Syntaxbaum aus Textdarstellung wieder aufbauen
--   procedure liesBaum (baum : out Syntaxbaum; datei : Textdatei);

end Baum;