with Grammatiken; use Grammatiken;
with text_io; use text_io;
with ada.strings.unbounded;
use ada.strings.unbounded;
with Mengen; use Mengen;


procedure GrammatikTester is

   G1 : Grammatik := new Grammatik_Rec (1,3);  -- ein NT und zwei Ts 
   G2 : Grammatik := new Grammatik_Rec (3,9); -- drei NTs und sechs Ts  
   G3 : Grammatik := new Grammatik_Rec (6,9);   -- sechs NTs und drei Ts
      
   leer : IntegerArray (1..0); -- fr die leeren Alternative 
      
begin
   
   G1.name := to_Unbounded_String("Klammergebirge");

   -- NT = {S}
   G1.text(1) := To_Unbounded_String("S");
   
   -- T  = {(,)}
   G1.text(2) := To_Unbounded_String("(");
   G1.text(3) := To_Unbounded_String(")");
   
   G1.regeln(1) := new Regel_Rec(2); -- S hat 2 Alternativen
   
   -- S -> ( S ) S
   G1.regeln(1).alt(1) := 
      new Alternativen_Rec'(4,1,IntegerArray'(2,1,3,1),0,
                            LeerMengeAuf(2,4),
                            LeerMengeAuf(2,4));  
   
   -- S -> epsilon 
   G1.regeln(1).alt(2) := 
      new Alternativen_Rec'(0,1,leer,0,
                            LeerMengeAuf(2,4),
                            LeerMengeAuf(2,4));
         
   drucke (G1);    
   new_line;
   
   bestimmeInfos (G1, True);
   druckeInfos (G1);  
   new_line(2);
   
   ---------------------------------------------------------------------------
    
   G2.name := to_Unbounded_String("Arithmetische Ausdruecke");

   -- NT = {Expression, Term, Factor}
   G2.text(1) := To_Unbounded_String("Expression");   
   G2.text(2) := To_Unbounded_String("Term");
   G2.text(3) := To_Unbounded_String("Factor");
   
   -- T  = {(, ), +, *, name, zahl}
   G2.text(4) := To_Unbounded_String("(");   
   G2.text(5) := To_Unbounded_String(")");
   G2.text(6) := To_Unbounded_String("+");   
   G2.text(7) := To_Unbounded_String("*");   
   G2.text(8) := To_Unbounded_String("name");
   G2.text(9) := To_Unbounded_String("zahl");
   
   G2.regeln(1) := new Regel_Rec(1); -- Expression hat eine Alternative
   
   -- Expression -> Term { + Term }
   G2.regeln(1).alt(1) := 
      new Alternativen_Rec'(3,1,IntegerArray'(2,6,2),2,
                            LeerMengeAuf(4,10),
                            LeerMengeAuf(4,10));  
   
   G2.regeln(2) := new Regel_Rec(1); -- Term hat eine Alternative
   
   -- Term -> Factor { * Factor }
   G2.regeln(2).alt(1) := 
      new Alternativen_Rec'(3,2,IntegerArray'(3,7,3),2,
                            LeerMengeAuf(4,10),
                            LeerMengeAuf(4,10));
   
   G2.regeln(3) := new Regel_Rec(3); -- Expression hat drei Alternativen
   
   -- Factor -> ( Expression )
   G2.regeln(3).alt(1) := 
      new Alternativen_Rec'(3,3,IntegerArray'(4,1,5),0,
                            LeerMengeAuf(4,10),
                            LeerMengeAuf(4,10)); 
     
   -- Factor -> name
   G2.regeln(3).alt(2) := 
      new Alternativen_Rec'(1,3,IntegerArray'(1=>8),0,
                            LeerMengeAuf(4,10),
                            LeerMengeAuf(4,10)); 
     
   -- Factor -> zahl
   G2.regeln(3).alt(3) := 
      new Alternativen_Rec'(1,3,IntegerArray'(1=>9),0,
                            LeerMengeAuf(4,10),
                            LeerMengeAuf(4,10));
            
   drucke (G2);    
   new_line;
   
   bestimmeInfos (G2, True);
   druckeInfos (G2); 
   new_line(2); 
      
   ---------------------------------------------------------------------------
      
   G3.name := to_Unbounded_String("Kunstbeispiel mit vielen epsilons");

   -- NT = {S, A, B, C, D, E}
   G3.text(1) := To_Unbounded_String("S");
   G3.text(2) := To_Unbounded_String("A");   
   G3.text(3) := To_Unbounded_String("B");
   G3.text(4) := To_Unbounded_String("C");  
   G3.text(5) := To_Unbounded_String("D");
   G3.text(6) := To_Unbounded_String("E");

   -- T  = {a, b, d}
   G3.text(7) := To_Unbounded_String("a");   
   G3.text(8) := To_Unbounded_String("b");   
   G3.text(9) := To_Unbounded_String("d");
      
   G3.regeln(1) := new Regel_Rec(2);  -- S hat zwei Alternativen
   
   -- S -> C C
   G3.regeln(1).alt(1) := 
      new Alternativen_Rec'(2,1,IntegerArray'(4,4),0,
                            LeerMengeAuf(7,10),
                            LeerMengeAuf(7,10)); 
     
   -- S -> A E
   G3.regeln(1).alt(2) := 
      new Alternativen_Rec'(2,1,IntegerArray'(2,6),0,
                            LeerMengeAuf(7,10),
                            LeerMengeAuf(7,10));
      
   G3.regeln(2) := new Regel_Rec(2);  -- A hat zwei Alternativen
   
   -- A -> B D
   G3.regeln(2).alt(1) := 
      new Alternativen_Rec'(2,2,IntegerArray'(3,5),0,
                            LeerMengeAuf(7,10),
                            LeerMengeAuf(7,10)); 
     
   -- A -> S C
   G3.regeln(2).alt(2) := 
      new Alternativen_Rec'(2,2,IntegerArray'(1,4),0,
                            LeerMengeAuf(7,10),
                            LeerMengeAuf(7,10));
      
   G3.regeln(3) := new Regel_Rec(2);  -- B hat zwei Alternativen
   
   -- B -> a C
   G3.regeln(3).alt(1) := 
      new Alternativen_Rec'(2,3,IntegerArray'(7,4),0,
                            LeerMengeAuf(7,10),
                            LeerMengeAuf(7,10)); 
     
   -- B -> C S A
   G3.regeln(3).alt(2) := 
      new Alternativen_Rec'(3,3,IntegerArray'(4,1,2),0,
                            LeerMengeAuf(7,10),
                            LeerMengeAuf(7,10));
   
   G3.regeln(4) := new Regel_Rec(1); -- C hat eine Alternative
   
   -- C -> epsilon
   G3.regeln(4).alt(1) := 
      new Alternativen_Rec'(0,4,leer,0,
                            LeerMengeAuf(7,10),
                            LeerMengeAuf(7,10));
      
   G3.regeln(5) := new Regel_Rec(2);  -- D hat zwei Alternativen
   
   -- D -> d
   G3.regeln(5).alt(1) := 
      new Alternativen_Rec'(1,5,IntegerArray'(1=>9),0,
                            LeerMengeAuf(7,10),
                            LeerMengeAuf(7,10)); 
     
   -- D -> D S 
   G3.regeln(5).alt(2) := 
      new Alternativen_Rec'(2,5,IntegerArray'(5,1),0,
                            LeerMengeAuf(7,10),
                            LeerMengeAuf(7,10));
 
   
   G3.regeln(6) := new Regel_Rec(1); -- E hat eine Alternative
   
   -- E -> b E b
   G3.regeln(6).alt(1) := 
      new Alternativen_Rec'(3,6,IntegerArray'(8,6,8),0,
                            LeerMengeAuf(7,10),
                            LeerMengeAuf(7,10));
           
   drucke (G3);  
   new_line; 
   
   bestimmeInfos (G3, True); 
   druckeInfos (G3);  
   new_line(2);  

end GrammatikTester ;
