with ada.strings.unbounded;
use ada.strings.unbounded;
with text_io; use text_io;
with Mengen; use Mengen;
with Relationen; use Relationen;

package body Grammatiken is

   function txt (Zeichen : Positive; G : Grammatik) return String is
   begin
      if Zeichen > G.ende then
         return "????";
      elsif Zeichen = G.ende then
         return "#";
      else
         return to_String(G.text(Zeichen));
      end if;
   end txt;
   
   ---------------------------------------------------------------- 
   
   procedure drucke (G : Grammatik) is
      kopf : Unbounded_String := to_Unbounded_String("Grammatik '"  
                                 & to_String(G.name) 
                                 & "':");
   
      function txt (Zeichen : Positive) return String is
      begin
         return txt(Zeichen,G);
      end txt;
   begin
      put_line(to_String(kopf));
      put_line(to_String(length(kopf)*"=")); new_line;
      put_line ("Nichtterminale:" ); new_line;
      for i in 1 .. G.anzNT loop
         put_line ("   " & txt(i));
      end loop; new_line;
      put_line ("Terminale:" ); new_line;
      for i in G.anzNT+1 .. G.anzV loop
         put_line ("   " & txt(i));
      end loop; new_line;
      put_line ("Regeln:" ); new_line;
      for i in 1 .. G.anzNT loop
         for a in 1 .. G.regeln(i).Anzahl loop
            put ("   " & txt(i) & " -> "); 
            if G.regeln(i).alt(a).wdh = 0 then
               for z in 1 .. G.regeln(i).alt(a).Laenge loop
                  put (txt(G.regeln(i).alt(a).rhs(z)) & " ");
               end loop;
            else
               for z in 1 .. G.regeln(i).alt(a).wdh-1 loop
                  put (txt(G.regeln(i).alt(a).rhs(z)) & " ");
               end loop;
               put ("{ ");
               for z in G.regeln(i).alt(a).wdh .. G.regeln(i).alt(a).Laenge loop
                  put (txt(G.regeln(i).alt(a).rhs(z)) & " ");
               end loop;               
               put ("}");
            end if;
            new_line; -- nach Alternative in neue Zeile
         end loop;
         new_line; -- Leerzeile zwischen Regeln
      end loop;
   end drucke;
   
   ----------------------------------------------------------------
   
   procedure bestimmeInfos (G : in out Grammatik;
                            ablauf : Boolean := False) is
      geaendert : Boolean := True;
      allesEps  : Boolean;
      i         : Integer := 0;
      altesEps  : Menge;  
      Anfang    : Relation := LeerRelationAuf (1,G.anzV+1); 
      Ende      : Relation := LeerRelationAuf (1,G.anzV+1); 
      Neben     : Relation := LeerRelationAuf (1,G.anzV+1);
      AnfangH   : Relation;
      zei       : Integer;
            
      procedure druckeEps (i : in out Integer) is 
      begin
         put("G.eps(" & Integer'image(i) & " )  =  {");
         for e in von(G.eps) .. bis(G.eps) loop
            if LiegtIn(e, G.eps) then
               put(" " & txt(e,G));
            end if;
         end loop;
         put_line(" }");
         i := i+1;
      end druckeEps;
   
      procedure druckeRelation (R : Relation; G : Grammatik) is
      begin
         for quelle in von(R) .. bis(R) loop
            put_line(txt(quelle,G) & " :"); 
            put("   ");
            for ziel in von(R) .. bis(R) loop
               if LiegtIn(quelle,ziel,R) then
                  put(txt(ziel,G) & "  ");
               end if;
            end loop; new_line;
         end loop; new_line;
      end druckeRelation;
      
      procedure berechneAnf (R : in out Relation; G : Grammatik) is
         istEps : Boolean;
         zei    : Integer;
      begin
         for z in 1 .. G.anzV+1 loop -- reflexive Hlle hinzu (also +Id)
            Eintrag (z,z,R); 
         end loop;
         for rg in 1 .. G.anzNT loop
            for a in 1 .. G.regeln(rg).Anzahl loop 
               istEps := True;
               for z in G.regeln(rg).alt(a).rhs'range loop
                  zei := G.regeln(rg).alt(a).rhs(z);
                  Eintrag(G.regeln(rg).alt(a).lhs,zei,R);
                  istEps := zei <= G.anzNT and then
                            LiegtIn (zei, G.eps);
                  exit when not istEps;
               end loop;
            end loop;
         end loop;
      end berechneAnf;
      
      procedure berechneEnd (R : in out Relation; G : Grammatik) is
         istEps : Boolean;
         zei    : Integer;
      begin
         for z in 1 .. G.anzV+1 loop -- reflexive Hlle hinzu (also +Id)
            Eintrag (z,z,R); 
         end loop;
         for rg in 1 .. G.anzNT loop
            for a in 1 .. G.regeln(rg).Anzahl loop 
               istEps := True;
               for z in reverse G.regeln(rg).alt(a).rhs'range loop
                  zei := G.regeln(rg).alt(a).rhs(z);
                  Eintrag(G.regeln(rg).alt(a).lhs,zei,R);
                  istEps := zei <= G.anzNT and then
                            LiegtIn (zei, G.eps);
                  exit when not istEps;
               end loop;
               -- wenn der optionale wdh-Teil entfllt, 
               -- dann liegt das Ende weiter links
               if G.regeln(rg).alt(a).wdh > 0 then
                  istEps := True;
                  for z in reverse 1 .. G.regeln(rg).alt(a).wdh-1 loop
                     zei := G.regeln(rg).alt(a).rhs(z);
                     Eintrag(G.regeln(rg).alt(a).lhs,zei,R);
                     istEps := zei <= G.anzNT and then
                               LiegtIn (zei, G.eps);
                     exit when not istEps;
                  end loop;
               end if;
            end loop;
         end loop;
      end berechneEnd;
      
      procedure berechneNeb (R : in out Relation; G : Grammatik) is
         istEps      : Boolean;
         zeiLi,zeiRe : Integer;
      begin
         for rg in 1 .. G.anzNT loop
            for a in 1 .. G.regeln(rg).Anzahl loop  
               for li in G.regeln(rg).alt(a).rhs'range loop
                  istEps := True;
                  zeiLi := G.regeln(rg).alt(a).rhs(li);
                  for re in li+1 .. G.regeln(rg).alt(a).rhs'last loop
                     zeiRe := G.regeln(rg).alt(a).rhs(re);
                     Eintrag(zeiLi,zeiRe,R);
                     istEps := zeiRe <= G.anzNT and then
                               LiegtIn (zeiRe, G.eps);
                     exit when not istEps;
                  end loop;
               end loop;
            end loop;
         end loop;
      end berechneNeb;

   begin
   
      -- Berechnung der epsilon-Symbole:
   
      if ablauf then  
         put_line("Berechnung der epsilon-Symbole:"); 
         put_line("===============================");
         new_line; 
         druckeEps(i); 
      end if; 
      while geaendert loop
         altesEps := kopieVon(G.eps);
         geaendert := False;
         for r in G.regeln'range loop
            for a in 1 .. G.regeln(r).Anzahl loop
               if not LiegtIn (G.regeln(r).alt(a).lhs, altesEps) then
                  allesEps := True;
                  for z in G.regeln(r).alt(a).rhs'range loop
                     allesEps := G.regeln(r).alt(a).rhs(z) <= G.anzNT and then
                                 LiegtIn (G.regeln(r).alt(a).rhs(z), altesEps);
                     exit when not allesEps;
                  end loop;
                  if allesEps then
                     geaendert := True;
                     if ablauf then
                        put_line("hinzu kommt " & txt(G.regeln(r).alt(a).lhs,G));
                     end if;
                     Eintrag (G.regeln(r).alt(a).lhs, G.eps);
                  end if;
               end if;
            end loop; 
         end loop;
         if ablauf and geaendert then druckeEps(i); end if;
      end loop;
      if ablauf then 
         put_line("Berechnung der Epsilonsymbole abgeschlossen"); 
         new_line(2);
      end if;
   
      -- Berechnung der Relation Anfang:  
      berechneAnf (Anfang, G); 
      if ablauf then  
         put_line("Relation Anfang+Id:");  
         put_line("===================");
         new_line;
         druckeRelation (Anfang, G); 
      end if;
      
      -- Berechnung der Relation Ende: 
      berechneEnd (Ende, G); 
      if ablauf then  
         put_line("Relation Ende+Id:");  
         put_line("=================");
         new_line;
         druckeRelation (Ende, G); 
      end if;
   
      -- Berechnung der Relation Neben: 
      berechneNeb (Neben, G); 
      if ablauf then  
         put_line("Relation Neben:");  
         put_line("===============");
         new_line;
         druckeRelation (Neben, G);
      end if;
   
      -- Berechnung der first-Mengen:
      AnfangH := Warshall(Anfang); 
      if ablauf then   
         put_line("Berechnung der first-Mengen:");  
         put_line("============================");
         new_line; 
         put_line("Relation Warshall(Anfang+Id):");  
         put_line("-----------------------------");
         new_line;
         druckeRelation (AnfangH, G);
      end if; 
      
      for nt in 1 .. G.anzNT loop
         for t in G.anzNT+1 .. G.anzV+1 loop
            if LiegtIn (nt,t,AnfangH) then
               Eintrag (t,G.firstM(nt));
            end if;
         end loop;
      end loop; 
      
      if ablauf then   
         put_line("daraus die first-Mengen:");   
         put_line("------------------------");
         new_line;
         for nt in 1 .. G.anzNT loop
            put_line("first(" & txt(nt,G) & "):");
            put("   {");
            for t in G.anzNT+1 .. G.anzV+1 loop
               if LiegtIn (t,G.firstM(nt)) then
                  put(" " & txt(t,G));
               end if;
            end loop;
            put_line("  }");
         end loop;
         new_line;
      end if;
         
      -- Berechnung der follow-Mengen:

      Eintrag (1,G.anzV+1,Neben); --> (S,#) nach Neben !!!
      AnfangH := Warshall(transponiere(Ende)) * Neben * AnfangH;
      
      for nt in 1 .. G.anzNT loop
         for t in G.anzNT+1 .. G.anzV+1 loop
            if LiegtIn (nt,t,AnfangH) then
               Eintrag (t,G.follow(nt));
            end if;
         end loop;
      end loop; 
      
      if ablauf then     
         put_line("Berechnung der follow-Mengen:");    
         put_line("=============================");  
         new_line;   
         put_line("als Relationenprodukt");  
         new_line;  
         put_line("Warshall(transponiere(Ende+Id)) * (Neben+(S,#)) * Warshall(Anfang+Id)");   
         new_line;  
         put_line("ergibt:");
         new_line;
         for nt in 1 .. G.anzNT loop
            put_line("follow(" & txt(nt,G) & "):");
            put("   {");
            for t in G.anzNT+1 .. G.anzV+1 loop
               if LiegtIn (t,G.follow(nt)) then
                  put(" " & txt(t,G));
               end if;
            end loop;
            put_line("  }");
         end loop;
         new_line;
      end if;
   
      -- Berechnung der LL(1)-Vorschaumengen:
      
      for rg in 1 .. G.anzNT loop
         for a in 1 .. G.regeln(rg).Anzahl loop
            -- berechnet erst die normale LL(1)-Vorschau ...
            allesEps := True;
            G.regeln(rg).alt(a).LL := LeerMengeAuf(G.anzNT+1,G.anzV+1);
            for z in 1 .. G.regeln(rg).alt(a).Laenge loop
               zei := G.regeln(rg).alt(a).rhs(z);
               if zei > G.anzNT then
                  Eintrag(zei,G.regeln(rg).alt(a).LL);
                  allesEps := False;  -- terminales Zeichen 
               else
                  Hinzu(G.regeln(rg).alt(a).LL,G.firstM(zei));
                  allesEps := LiegtIn(zei,G.eps);
               end if;
               exit when not allesEps;
            end loop;
            if allesEps then 
               Hinzu(G.regeln(rg).alt(a).LL,
                     G.follow(G.regeln(rg).alt(a).lhs));
            end if;
            -- dann noch die Vorschau fr die wdh-Anteile
            if G.regeln(rg).alt(a).wdh > 0 then
               allesEps := True;
               G.regeln(rg).alt(a).LL2 := LeerMengeAuf(G.anzNT+1,G.anzV+1);
               for z in G.regeln(rg).alt(a).wdh .. G.regeln(rg).alt(a).Laenge loop
                  zei := G.regeln(rg).alt(a).rhs(z);
                  if zei > G.anzNT then
                     Eintrag(zei,G.regeln(rg).alt(a).LL2);
                     allesEps := False;  -- terminales Zeichen 
                  else
                     Hinzu(G.regeln(rg).alt(a).LL2,G.firstM(zei));
                     allesEps := LiegtIn(zei,G.eps);
                  end if;
                  exit when not allesEps;
               end loop;
               if allesEps then 
                  Hinzu(G.regeln(rg).alt(a).LL2,
                        G.follow(G.regeln(rg).alt(a).lhs));
               end if;
            end if;
         end loop;
      end loop;
      
      if ablauf then     
         put_line("Berechnung der LL-Vorschau-Mengen:");    
         put_line("=================================="); 
         new_line;  
         put_line("ergibt:"); 
         new_line;      
         for i in 1 .. G.anzNT loop
            for a in 1 .. G.regeln(i).Anzahl loop
               put ("zu " & txt(i,G) & " -> "); 
               if G.regeln(i).alt(a).wdh = 0 then
                  for z in 1 .. G.regeln(i).alt(a).Laenge loop
                     put (txt(G.regeln(i).alt(a).rhs(z),G) & " ");
                  end loop;
               else
                  for z in 1 .. G.regeln(i).alt(a).wdh-1 loop
                     put (txt(G.regeln(i).alt(a).rhs(z),G) & " ");
                  end loop;
                  put ("{ ");
                  for z in G.regeln(i).alt(a).wdh .. G.regeln(i).alt(a).Laenge loop
                     put (txt(G.regeln(i).alt(a).rhs(z),G) & " ");
                  end loop;               
                  put ("}");
               end if;
               new_line;
               put("   {");
               for t in G.anzNT+1 .. G.anzV+1 loop
                  if LiegtIn (t,G.regeln(i).alt(a).LL) then
                     put(" " & txt(t,G));
                  end if;
               end loop;
               put_line("  }");
               if G.regeln(i).alt(a).wdh > 0 then
                  put("   {");
                  for t in G.anzNT+1 .. G.anzV+1 loop
                     if LiegtIn (t,G.regeln(i).alt(a).LL2) then
                        put(" " & txt(t,G));
                     end if;
                  end loop;
                  put_line("  }");
               end if;
               new_line; -- nach Alternative in neue Zeile
            end loop;
            new_line; -- Leerzeile zwischen Regeln
         end loop;
         new_line;
      end if;
      
   end bestimmeInfos;
   
   ---------------------------------------------------------------- 
   
   procedure druckeInfos (G : Grammatik) is
   
   begin
    
      new_line;
      put_line("Grammatik-Infos:");
      put_line("++++++++++++++++"); 
      new_line;
      
      put_line("Epsilonsymbole:");
      put("   {");
      for e in von(G.eps) .. bis(G.eps) loop
         if LiegtIn(e, G.eps) then
            put(" " & txt(e,G));
         end if;
      end loop;
      put_line(" }"); 
      new_line;
        
      put_line("first-Mengen:"); 
      new_line;
      for nt in 1 .. G.anzNT loop
         put_line("first(" & txt(nt,G) & "):");
         put("   {");
         for t in G.anzNT+1 .. G.anzV+1 loop
            if LiegtIn (t,G.firstM(nt)) then
               put(" " & txt(t,G));
            end if;
         end loop;
         put_line("  }");
      end loop;
      new_line;
    
      put_line("follow-Mengen:"); 
      new_line;
      for nt in 1 .. G.anzNT loop
         put_line("follow(" & txt(nt,G) & "):");
         put("   {");
         for t in G.anzNT+1 .. G.anzV+1 loop
            if LiegtIn (t,G.follow(nt)) then
               put(" " & txt(t,G));
            end if;
         end loop;
         put_line("  }");
      end loop;
      new_line;
    
      put_line("LL-Vorschau-Mengen:"); 
      new_line;      
      for i in 1 .. G.anzNT loop
         for a in 1 .. G.regeln(i).Anzahl loop
            put ("zu " & txt(i,G) & " -> "); 
            if G.regeln(i).alt(a).wdh = 0 then
               for z in 1 .. G.regeln(i).alt(a).Laenge loop
                  put (txt(G.regeln(i).alt(a).rhs(z),G) & " ");
               end loop;
            else
               for z in 1 .. G.regeln(i).alt(a).wdh-1 loop
                  put (txt(G.regeln(i).alt(a).rhs(z),G) & " ");
               end loop;
               put ("{ ");
               for z in G.regeln(i).alt(a).wdh .. G.regeln(i).alt(a).Laenge loop
                  put (txt(G.regeln(i).alt(a).rhs(z),G) & " ");
               end loop;               
               put ("}");
            end if;
            new_line;
            put("   {");
            for t in G.anzNT+1 .. G.anzV+1 loop
               if LiegtIn (t,G.regeln(i).alt(a).LL) then
                  put(" " & txt(t,G));
               end if;
            end loop;
            put_line("  }");
            if G.regeln(i).alt(a).wdh > 0 then
               put("   {");
               for t in G.anzNT+1 .. G.anzV+1 loop
                  if LiegtIn (t,G.regeln(i).alt(a).LL2) then
                     put(" " & txt(t,G));
                  end if;
               end loop;
               put_line("  }");
            end if;
            new_line; -- nach Alternative in neue Zeile
         end loop;
         new_line; -- Leerzeile zwischen Regeln
      end loop;

   end druckeInfos;
   
   
end Grammatiken;