with Text_IO; use Text_IO; 

package body Relationen is

   package Int_IO  is new Integer_IO(Integer); 
   use Int_IO; 

   type Adjazenz is array (Integer range <>, Integer range <>) 
      of Boolean; 
   
   type Relation_Rec (von, bis : Integer) is record 
      matrix : Adjazenz (von .. bis, von .. bis) := 
        (others => (others => false));
   end record;
  
   ------------------------------------------------------------- 

   -- Erstellt eine leere Relation auf der Grundmenge [von .. bis].
   function LeerRelationAuf (von, bis : Integer) return Relation is
   begin
      return new Relation_Rec (von, bis);
   end LeerRelationAuf; 
  
   ------------------------------------------------------------- 
   
   -- Trgt ein neues Paar (a,b) in die Relation R ein. 
   procedure Eintrag (a,b : Integer; R : in out Relation) is
   begin
      if not ((a in R.matrix'range(1)) and 
              (b in R.matrix'range(2))) then
         raise ElementNichtInGrundmenge;
      else 
         R.matrix(a,b) := True;
      end if;
   end Eintrag;

   -- Lscht ein Paar (a,b) aus der Relation R.
   procedure Loesche (a,b : Integer; R : in out Relation) is
   begin
      if not ((a in R.matrix'range(1)) and 
              (b in R.matrix'range(2))) then
         raise ElementNichtInGrundmenge;
      else 
         R.matrix(a,b) := False;
      end if;
   end Loesche; 
  
   ------------------------------------------------------------- 
   
   -- Trgt alle Paare in die Relation R ein. 
   procedure Eintragen (Paare : PaarArray; M : in out Relation) is
   begin
      for i in Paare'range loop
         Eintrag (Paare(i).a, Paare(i).b, M);
      end loop;
   end Eintragen;


   -- Prft, ob das Paar (a,b) in R liegt.
   function LiegtIn (a,b : Integer; R : Relation) return Boolean is
   begin
      if not ((a in R.matrix'range(1)) and 
              (b in R.matrix'range(2))) then
         raise ElementNichtInGrundmenge;
      else
         return R.matrix(a,b);
      end if;
   end LiegtIn; 
  
   ------------------------------------------------------------- 

--   -- Prft, ob eine Relation R leer ist.
--   function IstLeer (R : Relation) return Boolean; 
  
   ------------------------------------------------------------- 

   -- Konverse Relation zu R.
   function transponiere(R : Relation) return Relation is
      T : Relation := LeerRelationAuf (R.von, R.bis);
   begin
      for i in R.von .. R.bis loop
         for j in R.von .. R.bis loop
             T.matrix(i,j) := R.matrix(j,i); 
         end loop;
      end loop; 
      return T;
   end transponiere;
  
   ------------------------------------------------------------- 

   -- Bildet die Vereinigungsmenge zweier Relationen.
   function "+" (R1, R2 : Relation) return Relation is
      T : Relation := LeerRelationAuf (R1.von, R1.bis);
   begin
      if (R1.von /= R2.von or R1.bis /= R2.bis) then
         raise GrundmengenVerschieden;
      else
         for i in R1.von .. R1.bis loop
            for j in R1.von .. R1.bis loop
               T.matrix(i,j) := 
                  R1.matrix(i,j) or R2.matrix(i,j); 
            end loop;
         end loop; 
      end if;
      return T;
   end "+"; 

   -- Bildet das Produkt zweier Relationen.
   function "*" (R1, R2 : Relation) return Relation is
      T : Relation := LeerRelationAuf (R1.von, R1.bis);
   begin
      if (R1.von /= R2.von or R1.bis /= R2.bis) then
         raise GrundmengenVerschieden;
      else
         for i in R1.von .. R1.bis loop
            for j in R1.von .. R1.bis loop
               T.matrix(i,j) := false;
               for k in R1.von .. R1.bis loop
                  T.matrix(i,j) := T.matrix(i,j) or 
                     (R1.matrix(i,k) and R2.matrix(k,j));
               end loop;
            end loop;
         end loop; 
      end if; 
      return T; 
   end "*";  
  
   -------------------------------------------------------------

   -- Kleinstes Element der Grundmenge.
   function von (R : Relation) return Integer is
   begin
      return R.von;
   end von; 

   -- Grtes Element der Grundmenge.
   function bis (R : Relation) return Integer is
   begin
      return R.bis;
   end bis;
  
   -------------------------------------------------------------

   -- Bildet die transitive Hlle einer Relation.
   function Warshall (R : Relation) return Relation is 
      T : Relation := LeerRelationAuf (R.von, R.bis) + R;
   begin
      for k in R.von .. R.bis loop
         for i in R.von .. R.bis loop
            if T.matrix(i,k) then
               for j in R.von .. R.bis loop
                  T.matrix(i,j) := 
                     T.matrix(i,j) or T.matrix(k,j);
               end loop;
            end if;
         end loop;
      end loop;
      return T; 
   end Warshall;
   
   -------------------------------------------------------------
   
   -- Nimmt Elemente von R2 zu R1 hinzu.
   procedure Hinzu (R1 : in out Relation; R2 : Relation) is
   begin 
      if (R1.von /= R2.von or R1.bis /= R2.bis) then
         raise GrundmengenVerschieden;
      else
         for i in R1.von .. R1.bis loop
            for j in R1.von .. R1.bis loop
               R1.matrix(i,j) := 
                  R1.matrix(i,j) or R2.matrix(i,j); 
            end loop;
         end loop; 
      end if;
   end Hinzu;
 
   -- Entfernt die Elemente von R2 aus R1.
   procedure Hinweg (R1 : in out Relation; R2 : Relation) is
   begin
      if (R1.von /= R2.von or R1.bis /= R2.bis) then
         raise GrundmengenVerschieden;
      else
         for i in R1.von .. R1.bis loop
            for j in R1.von .. R1.bis loop
               R1.matrix(i,j) := 
                  R1.matrix(i,j) and not R2.matrix(i,j); 
            end loop;
         end loop; 
      end if;
   end Hinweg;
  
   -------------------------------------------------------------
   
   -- Druckt R.
   procedure Drucke (R : Relation; 
                     Text : in String := "Adjazenzmatrix:")is
   begin
      new_line(2);   
      put_line(Text);
      new_line; 
      put("     ");
      for i in R.von .. R.bis loop put(i,3); end loop;
      new_line(2);
      for i in R.von .. R.bis loop
         put(i,3); put(" :");
         for j in R.von .. R.bis loop
            if R.matrix(i,j) then 
               put("  1");
            else 
               put("  0"); 
            end if;
         end loop;
         new_line;
      end loop;
      new_line; 
   end Drucke;
      
end Relationen;